@@ -10,11 +10,19 @@
 #include "../casamaskreader.h"
 #include "../fitsreader.h"
 #include "../image.h"
+#include "../ndppp.h"
+#include "../rmsimage.h"
 
+#include "../units/fluxdensity.h"
+
+#include "../wsclean/imagefilename.h"
 #include "../wsclean/imagingtable.h"
+#include "../wsclean/primarybeam.h"
 #include "../wsclean/wscleansettings.h"
 
-Deconvolution::Deconvolution(const class WSCleanSettings& settings) : _settings(settings), _autoMaskIsFinished(false)
+Deconvolution::Deconvolution(const class WSCleanSettings& settings) :
+	_settings(settings), _autoMaskIsFinished(false),
+	_beamSize(0.0), _pixelScaleX(0.0), _pixelScaleY(0.0)
 {
 }
 
@@ -36,15 +44,56 @@ void Deconvolution::Perform(const class ImagingTable& groupTable, bool& reachedM
 	residualSet.LoadAndAverage(*_residualImages);
 	modelSet.LoadAndAverage(*_modelImages);
 	
-	ImageBufferAllocator::Ptr integrated;
-	_imageAllocator->Allocate(_imgWidth * _imgHeight, integrated);
+	Image integrated(_imgWidth, _imgHeight, *_imageAllocator);
 	residualSet.GetLinearIntegrated(integrated.data());
-	double stddev = Image::StdDevFromMAD(integrated.data(), _imgWidth * _imgHeight);
-	Logger::Info << "Estimated standard deviation of background noise: " << stddev << " Jy\n";
-	if(_settings.autoDeconvolutionThreshold && (!_settings.autoMask || _autoMaskIsFinished))
-		_cleanAlgorithm->SetThreshold(std::max(stddev * _settings.autoDeconvolutionThresholdSigma, _settings.deconvolutionThreshold));
-	else if(_settings.autoMask && !_autoMaskIsFinished)
+	double stddev = integrated.StdDevFromMAD();
+	Logger::Info << "Estimated standard deviation of background noise: " << FluxDensity::ToNiceString(stddev) << '\n';
+	if(_settings.autoMask && _autoMaskIsFinished)
+	{
+		// When we are in the second phase of automasking, don't use
+		// the RMS background anymore
+		_cleanAlgorithm->SetRMSFactorImage(Image());
+	}
+	else {
+		if(!_settings.rmsBackgroundImage.empty())
+		{
+			Image rmsImage(_imgWidth, _imgHeight, *_imageAllocator);
+			FitsReader reader(_settings.rmsBackgroundImage);
+			reader.Read(rmsImage.data());
+			// Normalize the RMS image
+			stddev = rmsImage.Min();
+			Logger::Info << "Lowest RMS in image: " << FluxDensity::ToNiceString(stddev) << '\n';
+			if(stddev <= 0.0)
+				throw std::runtime_error("RMS image can only contain values > 0, but contains values <= 0.0");
+			for(double& value : rmsImage)
+				value = stddev / value;
+			_cleanAlgorithm->SetRMSFactorImage(std::move(rmsImage));
+		}
+		else if(_settings.rmsBackground)
+		{
+			Image rmsImage;
+			// TODO this should use full beam parameters
+			switch(_settings.rmsBackgroundMethod)
+			{
+				case WSCleanSettings::RMSWindow:
+					RMSImage::Make(rmsImage, integrated, _settings.rmsBackgroundWindow, _beamSize, _beamSize, 0.0, _pixelScaleX, _pixelScaleY);
+					break;
+				case WSCleanSettings::RMSAndMinimumWindow:
+					RMSImage::MakeWithNegativityLimit(rmsImage, integrated, _settings.rmsBackgroundWindow, _beamSize, _beamSize, 0.0, _pixelScaleX, _pixelScaleY);
+					break;
+			}
+			// Normalize the RMS image relative to the threshold so that Jy remains Jy.
+			stddev = rmsImage.Min();
+			Logger::Info << "Lowest RMS in image: " << FluxDensity::ToNiceString(stddev) << '\n';
+			for(double& value : rmsImage)
+				value = stddev / value;
+			_cleanAlgorithm->SetRMSFactorImage(std::move(rmsImage));
+		}
+	}
+	if(_settings.autoMask && !_autoMaskIsFinished)
 		_cleanAlgorithm->SetThreshold(std::max(stddev * _settings.autoMaskSigma, _settings.deconvolutionThreshold));
+	else if(_settings.autoDeconvolutionThreshold)
+		_cleanAlgorithm->SetThreshold(std::max(stddev * _settings.autoDeconvolutionThresholdSigma, _settings.deconvolutionThreshold));
 	integrated.reset();
 	
 	std::vector<ao::uvector<double>> psfVecs(groupTable.SquaredGroupCount());
@@ -64,7 +113,6 @@ void Deconvolution::Perform(const class ImagingTable& groupTable, bool& reachedM
 			else
 				algorithm.SetAutoMaskMode(true, false);
 		}
-		algorithm.SetUseFastSubMinorLoop(_settings.multiscaleFastSubMinorLoop);
 	}
 	else {
 		if(_settings.autoMask && _autoMaskIsFinished)
@@ -94,9 +142,13 @@ void Deconvolution::Perform(const class ImagingTable& groupTable, bool& reachedM
 		reachedMajorThreshold = true;
 	}
 	
-	residualSet.AssignAndStore(*_residualImages);
+	if(_settings.majorIterationCount != 0 && majorIterationNr >= _settings.majorIterationCount)
+	{
+		reachedMajorThreshold = false;
+		Logger::Info << "Maximum number of major iterations was reached: not continuing deconvolution.\n";
+	}
 	
-	SpectralFitter fitter(_settings.spectralFittingMode, _settings.spectralFittingTerms);
+	residualSet.AssignAndStore(*_residualImages);
 	modelSet.InterpolateAndStore(*_modelImages, _cleanAlgorithm->Fitter());
 }
 
@@ -105,12 +157,16 @@ void Deconvolution::FreeDeconvolutionAlgorithms()
 	_cleanAlgorithm.reset();
 }
 
-void Deconvolution::InitializeDeconvolutionAlgorithm(const ImagingTable& groupTable, PolarizationEnum psfPolarization, class ImageBufferAllocator* imageAllocator, size_t imgWidth, size_t imgHeight, double pixelScaleX, double pixelScaleY, size_t outputChannels, double beamSize, size_t threadCount)
+void Deconvolution::InitializeDeconvolutionAlgorithm(const ImagingTable& groupTable, PolarizationEnum psfPolarization, class ImageBufferAllocator* imageAllocator, size_t imgWidth, size_t imgHeight, double pixelScaleX, double pixelScaleY, double beamSize, size_t threadCount)
 {
 	_imageAllocator = imageAllocator;
 	_imgWidth = imgWidth;
 	_imgHeight = imgHeight;
 	_psfPolarization = psfPolarization;
+	_beamSize = beamSize;
+	_pixelScaleX = pixelScaleX;
+	_pixelScaleY = pixelScaleY;
+	_autoMaskIsFinished = false;
 	FreeDeconvolutionAlgorithms();
 	
 	_summedCount = groupTable.SquaredGroupCount();
@@ -146,6 +202,10 @@ void Deconvolution::InitializeDeconvolutionAlgorithm(const ImagingTable& groupTa
 		algorithm->SetMultiscaleScaleBias(_settings.multiscaleDeconvolutionScaleBias);
 		algorithm->SetMultiscaleNormalizeResponse(_settings.multiscaleNormalizeResponse);
 		algorithm->SetMultiscaleGain(_settings.multiscaleGain);
+		algorithm->SetShape(_settings.multiscaleShapeFunction);
+		algorithm->SetTrackComponents(_settings.saveSourceList);
+		algorithm->SetConvolutionPadding(_settings.multiscaleConvolutionPadding);
+		algorithm->SetUseFastSubMinorLoop(_settings.multiscaleFastSubMinorLoop);
 	}
 	else
 	{
@@ -162,9 +222,9 @@ void Deconvolution::InitializeDeconvolutionAlgorithm(const ImagingTable& groupTa
 	_cleanAlgorithm->SetThreadCount(threadCount);
 	_cleanAlgorithm->SetSpectralFittingMode(_settings.spectralFittingMode, _settings.spectralFittingTerms);
 	
-	ao::uvector<double> frequencies;
-	calculateDeconvolutionFrequencies(groupTable, frequencies);
-	_cleanAlgorithm->InitializeFrequencies(frequencies);
+	ao::uvector<double> frequencies, weights;
+	calculateDeconvolutionFrequencies(groupTable, frequencies, weights);
+	_cleanAlgorithm->InitializeFrequencies(frequencies, weights);
 	
 	if(!_settings.fitsDeconvolutionMask.empty())
 	{
@@ -181,8 +241,7 @@ void Deconvolution::InitializeDeconvolutionAlgorithm(const ImagingTable& groupTa
 				_cleanMask[i] = maskData[i]!=0.0;
 		}
 		_cleanAlgorithm->SetCleanMask(_cleanMask.data());
-	}
-	else if(!_settings.casaDeconvolutionMask.empty())
+	} else if(!_settings.casaDeconvolutionMask.empty())
 	{
 		if(_cleanMask.empty())
 		{
@@ -197,19 +256,78 @@ void Deconvolution::InitializeDeconvolutionAlgorithm(const ImagingTable& groupTa
 	}
 }
 
-void Deconvolution::calculateDeconvolutionFrequencies(const ImagingTable& groupTable, ao::uvector<double>& frequencies)
+void Deconvolution::calculateDeconvolutionFrequencies(const ImagingTable& groupTable, ao::uvector<double>& frequencies, ao::uvector<double>& weights)
 {
 	size_t deconvolutionChannels = _settings.deconvolutionChannelCount;
 	if(deconvolutionChannels == 0) deconvolutionChannels = _summedCount;
 	frequencies.assign(deconvolutionChannels, 0.0);
-	ao::uvector<size_t> weights(deconvolutionChannels, 0);
+	weights.assign(deconvolutionChannels, 0.0);
+	ao::uvector<size_t> counts(deconvolutionChannels, 0);
 	for(size_t i=0; i!=_summedCount; ++i)
 	{
-		double freq = groupTable.GetSquaredGroup(i)[0].CentralFrequency();
+		const ImagingTableEntry& entry = groupTable.GetSquaredGroup(i)[0];
+		double freq = entry.CentralFrequency();
 		size_t deconvolutionChannel = i * deconvolutionChannels / _summedCount;
 		frequencies[deconvolutionChannel] += freq;
-		weights[deconvolutionChannel]++;
+		Logger::Debug << "Weight: " << entry.imageWeight << '\n';
+		weights[deconvolutionChannel] += entry.imageWeight;
+		counts[deconvolutionChannel]++;
 	}
 	for(size_t i=0; i!=deconvolutionChannels; ++i)
-		frequencies[i] /= weights[i];
+		frequencies[i] /= counts[i];
+}
+
+void Deconvolution::SaveSourceList(const class ImagingTable& table, long double phaseCentreRA, long double phaseCentreDec) const
+{
+	std::string filename = _settings.prefixName + "-sources.txt";
+	if(_settings.useMultiscale)
+	{
+		MultiScaleAlgorithm& algorithm = static_cast<MultiScaleAlgorithm&>(*_cleanAlgorithm);
+		algorithm.GetComponentList().Write(filename, algorithm, _pixelScaleX, _pixelScaleY, phaseCentreRA, phaseCentreDec);
+	}
+	else {
+		_imageAllocator->FreeUnused();
+		ImageSet modelSet(&table, *_imageAllocator, _settings.deconvolutionChannelCount, _settings.squaredJoins, _imgWidth, _imgHeight);
+		modelSet.LoadAndAverage(*_modelImages);
+		ComponentList componentList(_imgWidth, _imgHeight, modelSet);
+		componentList.WriteSingleScale(filename, *_cleanAlgorithm, _pixelScaleX, _pixelScaleY, phaseCentreRA, phaseCentreDec);
+	}
+}
+
+void Deconvolution::SavePBSourceList(const class ImagingTable& table, long double phaseCentreRA, long double phaseCentreDec) const
+{
+	std::unique_ptr<ComponentList> list;
+	if(_settings.useMultiscale)
+	{
+		MultiScaleAlgorithm& algorithm = static_cast<MultiScaleAlgorithm&>(*_cleanAlgorithm);
+		list.reset(new ComponentList(algorithm.GetComponentList()));
+	}
+	else {
+		_imageAllocator->FreeUnused();
+		ImageSet modelSet(&table, *_imageAllocator, _settings.deconvolutionChannelCount, _settings.squaredJoins, _imgWidth, _imgHeight);
+		modelSet.LoadAndAverage(*_modelImages);
+		list.reset(new ComponentList(_imgWidth, _imgHeight, modelSet));
+	}
+	
+	for(size_t i=0; i!=table.SquaredGroupCount(); ++i)
+	{
+		const ImagingTableEntry entry = table.GetSquaredGroup(i).Front();
+		Logger::Debug << "Correcting source list of channel " << entry.outputChannelIndex << " for beam\n";
+		ImageFilename filename(entry.outputChannelIndex, entry.outputIntervalIndex);
+		filename.SetPolarization(entry.polarization);
+		PrimaryBeam pb(_settings);
+		PrimaryBeamImageSet beam(_imgWidth, _imgHeight, *_imageAllocator);
+		pb.Load(beam, filename);
+		list->CorrectForBeam(beam, entry.outputChannelIndex);
+	}
+	
+	std::string filename = _settings.prefixName + "-sources-pb.txt";
+	if(_settings.useMultiscale)
+	{
+		MultiScaleAlgorithm& algorithm = static_cast<MultiScaleAlgorithm&>(*_cleanAlgorithm);
+		list->Write(filename, algorithm, _pixelScaleX, _pixelScaleY, phaseCentreRA, phaseCentreDec);
+	}
+	else {
+		list->WriteSingleScale(filename, *_cleanAlgorithm, _pixelScaleX, _pixelScaleY, phaseCentreRA, phaseCentreDec);
+	}
 }
