@@ -15,7 +15,7 @@ public:
 	
 	void Perform(const class ImagingTable& groupTable, bool& reachedMajorThreshold, size_t majorIterationNr);
 	
-	void InitializeDeconvolutionAlgorithm(const ImagingTable& groupTable, PolarizationEnum psfPolarization, ImageBufferAllocator* imageAllocator, size_t imgWidth, size_t imgHeight, double pixelScaleX, double pixelScaleY, size_t outputChannels, double beamSize, size_t threadCount);
+	void InitializeDeconvolutionAlgorithm(const ImagingTable& groupTable, PolarizationEnum psfPolarization, ImageBufferAllocator* imageAllocator, size_t imgWidth, size_t imgHeight, double pixelScaleX, double pixelScaleY, double beamSize, size_t threadCount);
 	
 	void InitializeImages(class CachedImageSet& residuals, CachedImageSet& models, CachedImageSet& psfs)
 	{
@@ -30,8 +30,13 @@ public:
 	const DeconvolutionAlgorithm& GetAlgorithm() const { return *_cleanAlgorithm; }
 	
 	bool IsInitialized() const { return _cleanAlgorithm != 0; }
+	
+	void SaveSourceList(const class ImagingTable& table, long double phaseCentreRA, long double phaseCentreDec) const;
+	
+	void SavePBSourceList(const class ImagingTable& table, long double phaseCentreRA, long double phaseCentreDec) const;
+
 private:
-	void calculateDeconvolutionFrequencies(const ImagingTable& groupTable, ao::uvector<double>& frequencies);
+	void calculateDeconvolutionFrequencies(const ImagingTable& groupTable, ao::uvector<double>& frequencies, ao::uvector<double>& weights);
 	
 	const class WSCleanSettings& _settings;
 	
@@ -47,6 +52,7 @@ private:
 	ImageBufferAllocator* _imageAllocator;
 	CachedImageSet *_psfImages, *_modelImages, *_residualImages;
 	ao::uvector<bool> _autoMask;
+	double _beamSize, _pixelScaleX, _pixelScaleY;
 };
 
 #endif
