@@ -16,6 +16,11 @@ public:
 		_mode(mode), _nTerms(nTerms)
 	{ }
 	
+	SpectralFittingMode Mode() const
+	{
+		return _mode;
+	}
+	
 	void SetMode(SpectralFittingMode mode, size_t nTerms)
 	{
 		_mode = mode;
@@ -30,20 +35,21 @@ public:
 	
 	double Evaluate(const ao::uvector<double>& terms, double frequency) const;
 	
-	void SetFrequencies(const double* frequencies, size_t n)
+	void SetFrequencies(const double* frequencies, const double* weights, size_t n)
 	{
 		_frequencies.assign(frequencies, frequencies+n);
+		_weights.assign(weights, weights+n);
 	}
 	
 	size_t NTerms() const { return _nTerms; }
 	
-private:
-	double referenceFrequency() const {
+	double ReferenceFrequency() const {
 		return _frequencies[_frequencies.size()/2];
 	}
+private:
 	enum SpectralFittingMode _mode;
 	size_t _nTerms;
-	ao::uvector<double> _frequencies;
+	ao::uvector<double> _frequencies, _weights;
 };
 
 #endif
