@@ -1,6 +1,8 @@
 #include "commandline.h"
 
-#include "../angle.h"
+#include "../units/angle.h"
+#include "../units/fluxdensity.h"
+
 #include "../numberlist.h"
 #include "../wscversion.h"
 
@@ -56,13 +58,17 @@ void CommandLine::printHelp()
 		"-saveuv\n"
 		"   Save the gridded uv plane, i.e., the FFT of the residual image. The UV plane is complex, hence\n"
 		"   two images will be output: <prefix>-uv-real.fits and <prefix>-uv-imag.fits.\n"
+		"-save-uv-bins <psf limit>\n"
+		"   Saves a csv file with the gridded visibilities, including their weights.\n"
 		"-apply-primary-beam\n"
 		"   Calculate and apply the primary beam and save images for the Jones components, with weighting identical to the\n"
 		"   weighting as used by the imager. Only available for LOFAR.\n"
 		"-reuse-primary-beam\n"
-		"   If a primary beam image exists on disk, reuse those images (not implemented yet).\n"
+		"   If a primary beam image exists on disk, reuse those images.\n"
 		"-use-differential-lofar-beam\n"
 		"   Assume the visibilities have already been beam-corrected for the reference direction.\n"
+		"-save-psf-pb\n"
+		"   When applying beam correction, also save the primary-beam corrected PSF image.\n"
 		"\n"
 		"  ** WEIGHTING OPTIONS **\n"
 		"-weight <weightmode>\n"
@@ -199,7 +205,10 @@ void CommandLine::printHelp()
 		"\n"
 		"  ** DECONVOLUTION OPTIONS **\n"
 		"-niter <niter>\n"
-		"   Maximum number of clean iterations to perform. Default: 0\n"
+		"   Maximum number of clean iterations to perform. Default: 0 (=no cleaning)\n"
+		"-nmiter <nmiter>\n"
+		"   Maximum number of major clean (inversion/prediction) iterations. Default: 20."
+		"   A value of 0 means no limit.\n"
 		"-threshold <threshold>\n"
 		"   Stopping clean thresholding in Jy. Default: 0.0\n"
 		"-auto-threshold <sigma>\n"
@@ -207,6 +216,13 @@ void CommandLine::printHelp()
 		"-auto-mask <sigma>\n"
 		"   Construct a mask from found components and when a threshold of sigma is reached, continue\n"
 		"   cleaning with the mask down to the normal threshold. \n"
+		"-local-rms / -rms-background\n"
+		"   Instead of using a single RMS for auto thresholding/masking, use a spatially varying\n"
+		"   RMS image.\n"
+		"-local-rms-window / -rms-background-window\n"
+		"   Size of window for creating the RMS background map, in number of PSFs. Default: 25 psfs.\n"
+		"-local-rms-method / -rms-background-method\n"
+		"   Either 'rms' (default, uses sliding window RMS) or 'rms-with-min' (use max(window rms, 0.3 x window min)).\n"
 		"-gain <gain>\n"
 		"   Cleaning gain: Ratio of peak that will be subtracted in each iteration. Default: 0.1\n"
 		"-mgain <gain>\n"
@@ -234,6 +250,13 @@ void CommandLine::printHelp()
 		"   Sets a list of scales to use in multi-scale cleaning. If unset, WSClean will select the delta\n"
 		"   (zero) scale, scales starting at four times the synthesized PSF, and increase by a factor of\n"
 		"   two until the maximum scale is reached. Example: -multiscale-scales 0,5,12.5\n"
+		"-multiscale-shape <shape>\n"
+		"   Sets the shape function used during multi-scale clean. Either 'tapered-quadratic' (default) or 'gaussian'.\n"
+		"-multiscale-gain <gain>\n"
+		"   Size of step made in the subminor loop of multi-scale. Default currently 0.2, but shows sign of instability.\n"
+		"   A value of 0.1 might be more stable.\n"
+		"-multiscale-convolution-padding <padding>\n"
+		"   Size of zero-padding for convolutions during the multi-scale cleaning.\n"
 		"-no-multiscale-fast-subminor\n"
 		"   Disable the 'fast subminor loop' optimization, that will only search a part of the\n"
 		"   image during the multi-scale subminor loop. The optimization is on by default.\n"
@@ -250,6 +273,9 @@ void CommandLine::printHelp()
 		"-moresane-sl <sl1,sl2,...>\n"
 		"   MoreSane --sigmalevel setting for each major loop iteration. Useful to start at high\n"
 		"   levels and go down with subsequent loops, e.g. 20,10,5\n"
+		"-save-source-list\n"
+		"   Saves the found clean components as a BBS/NDPPP text sky model. This parameter \n"
+		"   enables Gaussian shapes during multi-scale cleaning (-multiscale-shape gaussian).\n"
 		"-cleanborder <percentage>\n"
 		"   Set the border size in which no cleaning is performed, in percentage of the width/height of the image.\n"
 		"   With an image size of 1000 and clean border of 1%, each border is 10 pixels. \n"
@@ -326,7 +352,7 @@ void CommandLine::printHeader()
 size_t CommandLine::parse_size_t(const char* param, const char* name)
 {
 	char* endptr;
-	errno=0;
+	errno = 0;
 	long v = strtol(param, &endptr, 0);
 	if(*endptr!=0 || endptr == param || errno!=0) {
 		std::ostringstream msg;
@@ -363,6 +389,9 @@ int CommandLine::Run(int argc, char* argv[])
 			printHeader();
 #ifdef HAVE_LOFAR_BEAM
 			Logger::Info << "LOFAR beam is available.\n";
+#endif
+#ifdef HAVE_IDG
+			Logger::Info << "IDG is available.\n";
 #endif
 			return 0;
 		}
@@ -466,10 +495,15 @@ int CommandLine::Run(int argc, char* argv[])
 			++argi;
 			settings.deconvolutionIterationCount = parse_size_t(argv[argi], "niter");
 		}
+		else if(param == "nmiter")
+		{
+			++argi;
+			settings.majorIterationCount = parse_size_t(argv[argi], "nmiter");
+		}
 		else if(param == "threshold")
 		{
 			++argi;
-			settings.deconvolutionThreshold = atof(argv[argi]);
+			settings.deconvolutionThreshold = FluxDensity::Parse(argv[argi], "threshold parameter", FluxDensity::Jansky);
 		}
 		else if(param == "auto-threshold")
 		{
@@ -483,6 +517,42 @@ int CommandLine::Run(int argc, char* argv[])
 			settings.autoMask = true;
 			settings.autoMaskSigma = atof(argv[argi]);
 		}
+		else if(param == "local-rms" || param == "rms-background")
+		{
+			settings.rmsBackground = true;
+			if(param == "rms-background")
+				deprecated(param, "local-rms");
+		}
+		else if(param == "local-rms-window" || param == "rms-background-window")
+		{
+			++argi;
+			settings.rmsBackground = true;
+			settings.rmsBackgroundWindow = atof(argv[argi]);
+			if(param == "rms-background-window")
+				deprecated(param, "local-rms-window");
+		}
+		else if(param == "local-rms-image" || param == "rms-background-image")
+		{
+			++argi;
+			settings.rmsBackground = true;
+			settings.rmsBackgroundImage = argv[argi];
+			if(param == "rms-background-image")
+				deprecated(param, "local-rms-image");
+		}
+		else if(param == "local-rms-method" || param == "rms-background-method")
+		{
+			++argi;
+			std::string method = argv[argi];
+			settings.rmsBackground = true;
+			if(method == "rms")
+				settings.rmsBackgroundMethod = WSCleanSettings::RMSWindow;
+			else if(method == "rms-with-min")
+				settings.rmsBackgroundMethod = WSCleanSettings::RMSAndMinimumWindow;
+			else
+				throw std::runtime_error("Unknown RMS background method specified");
+			if(param == "rms-background-method")
+				deprecated(param, "local-rms-method");
+		}
 		else if(param == "datacolumn")
 		{
 			++argi;
@@ -505,6 +575,10 @@ int CommandLine::Run(int argc, char* argv[])
 		{
 			settings.useDifferentialLofarBeam = true;
 		}
+		else if(param == "save-psf-pb")
+		{
+			settings.savePsfPb = true;
+		}
 		else if(param == "negative")
 		{
 			settings.allowNegativeComponents = true;
@@ -680,6 +754,21 @@ int CommandLine::Run(int argc, char* argv[])
 			++argi;
 			NumberList::ParseDoubleList(argv[argi], settings.multiscaleScaleList);
 		}
+		else if(param == "multiscale-shape")
+		{
+			++argi;
+			std::string shape = argv[argi];
+			if(shape == "tapered-quadratic")
+				settings.multiscaleShapeFunction = MultiScaleTransforms::TaperedQuadraticShape;
+			else if(shape == "gaussian")
+				settings.multiscaleShapeFunction = MultiScaleTransforms::GaussianShape;
+			else throw std::runtime_error("Unknown multiscale shape function given");
+		}
+		else if(param == "multiscale-convolution-padding")
+		{
+			++argi;
+			settings.multiscaleConvolutionPadding = atof(argv[argi]);
+		}
 		else if(param == "no-multiscale-fast-subminor")
 		{
 			settings.multiscaleFastSubMinorLoop = false;
@@ -694,6 +783,11 @@ int CommandLine::Run(int argc, char* argv[])
 			++argi;
 			settings.rankFilterSize = parse_size_t(argv[argi], "weighting-rank-filter-size");
 		}
+		else if(param == "save-source-list")
+		{
+			settings.saveSourceList = true;
+			settings.multiscaleShapeFunction = MultiScaleTransforms::GaussianShape;
+		}
 		else if(param == "cleanborder")
 		{
 			++argi;
@@ -919,6 +1013,8 @@ int CommandLine::Run(int argc, char* argv[])
 		else if(param == "use-idg")
 		{
 			settings.useIDG = true;
+			//settings.polarizations.clear();
+			//settings.polarizations.insert(Polarization::Instrumental);
 		}
 		else if(param == "no-dirty")
 		{
@@ -967,3 +1063,10 @@ int CommandLine::Run(int argc, char* argv[])
 	}
 	return 0;
 }
+
+void CommandLine::deprecated(const std::string& param, const std::string& replacement)
+{
+	Logger::Warn
+	  << "!!! WARNING: Parameter \'-" << param << "\' is deprecated and will be removed in a future version of WSClean.\n"
+		<< "!!!          Use parameter \'-" << replacement << "\' instead.\n";
+}
