@@ -1,5 +1,6 @@
 #include "wsclean.h"
 
+#include "binneduvoutput.h"
 #include "imageweightcache.h"
 #include "inversionalgorithm.h"
 #include "logger.h"
@@ -8,7 +9,8 @@
 #include "primarybeam.h"
 #include "imagefilename.h"
 
-#include "../angle.h"
+#include "../units/angle.h"
+
 #include "../application.h"
 #include "../areaset.h"
 #include "../dftpredictionalgorithm.h"
@@ -60,7 +62,7 @@ void WSClean::multiplyImage(double factor, double* image) const
 		image[i] *= factor;
 }
 
-void WSClean::imagePSF(const ImagingTableEntry& entry)
+void WSClean::imagePSF(ImagingTableEntry& entry)
 {
 	size_t channelIndex = entry.outputChannelIndex;
 	Logger::Info.Flush();
@@ -93,15 +95,17 @@ void WSClean::imagePSF(const ImagingTableEntry& entry)
 	
 	double bMaj, bMin, bPA;
 	determineBeamSize(bMaj, bMin, bPA, _gridder->ImageRealResult(), _gridder->BeamSize());
+	entry.imageWeight = _gridder->ImageWeight();
 	_infoPerChannel[channelIndex].theoreticBeamSize = _gridder->BeamSize();
 	_infoPerChannel[channelIndex].beamMaj = bMaj;
 	_infoPerChannel[channelIndex].beamMin = bMin;
 	_infoPerChannel[channelIndex].beamPA = bPA;
-	_infoPerChannel[channelIndex].weight = _gridder->ImageWeight();
+	_infoPerChannel[channelIndex].weight = entry.imageWeight;
 	_infoPerChannel[channelIndex].normalizationFactor = _gridder->NormalizationFactor();
 	_infoPerChannel[channelIndex].wGridSize = _gridder->WGridSize();
 	_infoPerChannel[channelIndex].visibilityCount = _gridder->GriddedVisibilityCount();
 	_infoPerChannel[channelIndex].effectiveVisibilityCount = _gridder->EffectiveGriddedVisibilityCount();
+	_infoPerChannel[channelIndex].visibilityWeightSum = _gridder->VisibilityWeightSum();
 		
 	if(_settings.isUVImageSaved)
 	{
@@ -319,7 +323,8 @@ void WSClean::initializeMFSImageWeights()
 					bool hasSelection = selectChannels(partSelection, msIndex, dataDescId, subTable.Front());
 					if(hasSelection)
 					{
-						PartitionedMS msProvider(_partitionedMSHandles[msIndex], ms.bands[dataDescId].partIndex, entry.polarization, dataDescId);
+						PolarizationEnum pol = _settings.useIDG ? Polarization::Instrumental : entry.polarization;
+						PartitionedMS msProvider(_partitionedMSHandles[msIndex], ms.bands[dataDescId].partIndex, pol, dataDescId);
 						_imageWeightCache->Weights().Grid(msProvider, partSelection);
 					}
 				}
@@ -331,7 +336,8 @@ void WSClean::initializeMFSImageWeights()
 		{
 			for(size_t d=0; d!=_msBands[i].DataDescCount(); ++d)
 			{
-				ContiguousMS msProvider(_settings.filenames[i], _settings.dataColumnName, _globalSelection, *_settings.polarizations.begin(), d, _settings.deconvolutionMGain != 1.0);
+				PolarizationEnum pol = _settings.useIDG ? Polarization::Instrumental : *_settings.polarizations.begin();
+				ContiguousMS msProvider(_settings.filenames[i], _settings.dataColumnName, _globalSelection, pol, d, _settings.deconvolutionMGain != 1.0);
 				_imageWeightCache->Weights().Grid(msProvider,  _globalSelection);
 				Logger::Info << '.';
 				Logger::Info.Flush();
@@ -454,7 +460,8 @@ void WSClean::RunClean()
 		
 		for(size_t groupIndex=0; groupIndex!=_imagingTable.IndependentGroupCount(); ++groupIndex)
 		{
-			runIndependentGroup(_imagingTable.GetIndependentGroup(groupIndex));
+			ImagingTable group = _imagingTable.GetIndependentGroup(groupIndex);
+			runIndependentGroup(group);
 		}
 
 		// Needs to be destructed before image allocator, or image allocator will report error caused by leaked memory
@@ -467,7 +474,11 @@ void WSClean::RunClean()
 				bool psfWasMade = (_settings.deconvolutionIterationCount > 0 || _settings.makePSF || _settings.makePSFOnly) && pol == _settings.polarizations.begin();
 				
 				if(psfWasMade)
+				{
 					makeMFSImage("psf.fits", intervalIndex, *pol, false, true);
+					if(_settings.savePsfPb)
+						makeMFSImage("psf-pb.fits", intervalIndex, *pol, false, true);
+				}
 				
 				if(!(*pol == Polarization::YX && _settings.polarizations.count(Polarization::XY)!=0) && !_settings.makePSFOnly)
 				{
@@ -565,10 +576,18 @@ void WSClean::RunPredict()
 		
 		if(_doReorder) performReordering(true);
 		
+		if(_settings.useIDG)
+			_gridder.reset(new IdgMsGridder());
+		else
+			_gridder.reset(new WSMSGridder(&_imageAllocator, _settings.threadCount, _settings.memFraction, _settings.absMemLimit));
+	
 		for(size_t groupIndex=0; groupIndex!=_imagingTable.SquaredGroupCount(); ++groupIndex)
 		{
 			predictGroup(_imagingTable.GetSquaredGroup(groupIndex));
 		}
+	
+		// Needs to be destructed before image allocator, or image allocator will report error caused by leaked memory
+		_gridder.reset();
 	}
 }
 
@@ -616,7 +635,20 @@ bool WSClean::selectChannels(MSSelection& selection, size_t msIndex, size_t data
 	}
 }
 
-void WSClean::runIndependentGroup(const ImagingTable& groupTable)
+double WSClean::minTheoreticalBeamSize(const ImagingTable& table) const
+{
+	double beam = 0.0;
+	for(size_t i=0; i!=table.EntryCount(); ++i)
+	{
+		const ImagingTableEntry& e = table[i];
+		const OutputChannelInfo& info = _infoPerChannel[e.outputChannelIndex];
+		if(std::isfinite(info.theoreticBeamSize) && (info.theoreticBeamSize < beam || beam == 0.0))
+			beam = info.theoreticBeamSize;
+	}
+	return beam;
+}
+
+void WSClean::runIndependentGroup(ImagingTable& groupTable)
 {
 	WSCFitsWriter writer(createWSCFitsWriter(groupTable.Front(), false));
 	_modelImages.Initialize(writer.Writer(), _settings.polarizations.size(), _settings.channelsOut, _settings.prefixName + "-model", _imageAllocator);
@@ -628,11 +660,11 @@ void WSClean::runIndependentGroup(const ImagingTable& groupTable)
 		
 	for(size_t joinedIndex=0; joinedIndex!=groupTable.EntryCount(); ++joinedIndex)
 	{
-		const ImagingTableEntry& entry = groupTable[joinedIndex];
+		ImagingTableEntry& entry = groupTable[joinedIndex];
 		runFirstInversion(entry);
 	}
 	
-	_deconvolution.InitializeDeconvolutionAlgorithm(groupTable, *_settings.polarizations.begin(), &_imageAllocator, _settings.trimmedImageWidth, _settings.trimmedImageHeight, _settings.pixelScaleX, _settings.pixelScaleY, _settings.channelsOut, _gridder->BeamSize(), _settings.threadCount);
+	_deconvolution.InitializeDeconvolutionAlgorithm(groupTable, *_settings.polarizations.begin(), &_imageAllocator, _settings.trimmedImageWidth, _settings.trimmedImageHeight, _settings.pixelScaleX, _settings.pixelScaleY, minTheoreticalBeamSize(groupTable), _settings.threadCount);
 
 	if(!_settings.makePSFOnly)
 	{
@@ -699,6 +731,14 @@ void WSClean::runIndependentGroup(const ImagingTable& groupTable)
 		
 		for(size_t joinedIndex=0; joinedIndex!=groupTable.EntryCount(); ++joinedIndex)
 			saveRestoredImagesForGroup(groupTable[joinedIndex]);
+		if(_settings.saveSourceList)
+    {
+			_deconvolution.SaveSourceList(groupTable, _gridder->PhaseCentreRA(), _gridder->PhaseCentreDec());
+			if(_settings.applyPrimaryBeam)
+			{
+				_deconvolution.SavePBSourceList(groupTable, _gridder->PhaseCentreRA(), _gridder->PhaseCentreDec());
+			}
+    }
 	}
 	
 	_imageAllocator.ReportStatistics();
@@ -760,6 +800,8 @@ void WSClean::saveRestoredImagesForGroup(const ImagingTableEntry& tableEntry) co
 		{
 			ImageFilename imageName = ImageFilename(currentChannelIndex, tableEntry.outputIntervalIndex);
 			_primaryBeam->CorrectImages(writer.Writer(), imageName, "image", _imageAllocator);
+			if(_settings.savePsfPb)
+				_primaryBeam->CorrectImages(writer.Writer(), imageName, "psf", _imageAllocator);
 			if(_settings.deconvolutionIterationCount != 0)
 			{
 				_primaryBeam->CorrectImages(writer.Writer(), imageName, "residual", _imageAllocator);
@@ -881,11 +923,6 @@ void WSClean::readEarlierModelImages(const ImagingTableEntry& entry)
 
 void WSClean::predictGroup(const ImagingTable& imagingGroup)
 {
-	if(_settings.useIDG)
-		_gridder.reset(new IdgMsGridder());
-	else
-		_gridder.reset(new WSMSGridder(&_imageAllocator, _settings.threadCount, _settings.memFraction, _settings.absMemLimit));
-	
 	_modelImages.Initialize(
 		createWSCFitsWriter(imagingGroup.Front(), false).Writer(),
 		_settings.polarizations.size(), 1, _settings.prefixName + "-model", _imageAllocator
@@ -912,17 +949,15 @@ void WSClean::predictGroup(const ImagingTable& imagingGroup)
 	Logger::Info << "Inversion: " << _inversionWatch.ToString() << ", prediction: " << _predictingWatch.ToString() << ", cleaning: " << _deconvolutionWatch.ToString() << '\n';
 	
 	_settings.prefixName = rootPrefix;
-	
-	// Needs to be destructed before image allocator, or image allocator will report error caused by leaked memory
-	_gridder.reset();
 }
 
 MSProvider* WSClean::initializeMSProvider(const ImagingTableEntry& entry, const MSSelection& selection, size_t filenameIndex, size_t dataDescId)
 {
+	PolarizationEnum pol = _settings.useIDG ? Polarization::Instrumental : entry.polarization;
 	if(_doReorder)
-		return new PartitionedMS(_partitionedMSHandles[filenameIndex], entry.msData[filenameIndex].bands[dataDescId].partIndex, entry.polarization, dataDescId);
+		return new PartitionedMS(_partitionedMSHandles[filenameIndex], entry.msData[filenameIndex].bands[dataDescId].partIndex, pol, dataDescId);
 	else
-		return new ContiguousMS(_settings.filenames[filenameIndex], _settings.dataColumnName, selection, entry.polarization, dataDescId, _settings.deconvolutionMGain != 1.0);
+		return new ContiguousMS(_settings.filenames[filenameIndex], _settings.dataColumnName, selection, pol, dataDescId, _settings.deconvolutionMGain != 1.0);
 }
 
 void WSClean::initializeCurMSProviders(const ImagingTableEntry& entry)
@@ -967,7 +1002,7 @@ void WSClean::clearCurMSProviders()
 	_currentPolMSes.clear();
 }
 
-void WSClean::runFirstInversion(const ImagingTableEntry& entry)
+void WSClean::runFirstInversion(ImagingTableEntry& entry)
 {
 	initializeCurMSProviders(entry);
 	initializeImageWeights(entry);
@@ -1008,7 +1043,8 @@ void WSClean::runFirstInversion(const ImagingTableEntry& entry)
 		// the info for this channel
 		if(isFirstPol)
 		{
-			_infoPerChannel[entry.outputChannelIndex].weight = _gridder->ImageWeight();
+			entry.imageWeight = _gridder->ImageWeight();
+			_infoPerChannel[entry.outputChannelIndex].weight = entry.imageWeight;
 			// If no PSF is made, also set the beam size. If the PSF was made, these would already be set
 			// after imaging the PSF.
 			if(!doMakePSF)
@@ -1159,7 +1195,6 @@ void WSClean::renderMFSImage(size_t intervalIndex, PolarizationEnum pol, bool is
 	residualReader.Read(image.data());
 	modelReader.Read(modelImage.data());
 	
-	//ModelRenderer renderer(_fitsWriter.RA(), _fitsWriter.Dec(), _settings.pixelScaleX, _settings.pixelScaleY, _fitsWriter.PhaseCentreDL(), _fitsWriter.PhaseCentreDM());
 	double beamMaj = _infoForMFS.beamMaj;
 	double beamMin, beamPA;
 	std::string beamStr;
@@ -1233,8 +1268,10 @@ void WSClean::saveUVImage(const double* image, PolarizationEnum pol, const Imagi
 		imagUV(_settings.trimmedImageWidth, _settings.trimmedImageHeight, _imageAllocator);
 	FFTResampler fft(_settings.trimmedImageWidth, _settings.trimmedImageHeight, _settings.trimmedImageWidth, _settings.trimmedImageHeight, 1, true);
 	fft.SingleFT(image, realUV.data(), imagUV.data());
-	realUV *= _infoPerChannel[entry.outputChannelIndex].normalizationFactor / sqrt(_settings.trimmedImageWidth * _settings.trimmedImageHeight);
-	imagUV *= _infoPerChannel[entry.outputChannelIndex].normalizationFactor / sqrt(_settings.trimmedImageWidth * _settings.trimmedImageHeight);
+	// There are two factors of 2 involved: one coming from
+	// SingleFT(), and one from the fact that normF excludes a factor of two.
+	realUV *= 4.0 * _infoPerChannel[entry.outputChannelIndex].normalizationFactor / sqrt(_settings.trimmedImageWidth * _settings.trimmedImageHeight);
+	imagUV *= 4.0 * _infoPerChannel[entry.outputChannelIndex].normalizationFactor / sqrt(_settings.trimmedImageWidth * _settings.trimmedImageHeight);
 	WSCFitsWriter writer(createWSCFitsWriter(entry, isImaginary));
 	writer.WriteUV(prefix+"-real.fits", realUV.data());
 	writer.WriteUV(prefix+"-imag.fits", imagUV.data());
@@ -1296,7 +1333,7 @@ void WSClean::makeImagingTable(size_t outputIntervalIndex)
 			for(size_t outChannelIndex=0; outChannelIndex!=_settings.channelsOut; ++outChannelIndex)
 			{
 				ImagingTableEntry freqTemplate;
-				makeImagingTableEntry(_inputChannelFrequencies, outChannelIndex, freqTemplate);
+				makeImagingTableEntry(_inputChannelFrequencies, outputIntervalIndex, outChannelIndex, freqTemplate);
 				
 				size_t localJGI = joinedGroupIndex;
 				addPolarizationsToImagingTable(localJGI, squaredGroupIndex, outChannelIndex, freqTemplate);
@@ -1309,7 +1346,7 @@ void WSClean::makeImagingTable(size_t outputIntervalIndex)
 			for(size_t outChannelIndex=0; outChannelIndex!=_settings.channelsOut; ++outChannelIndex)
 			{
 				ImagingTableEntry freqTemplate;
-				makeImagingTableEntry(_inputChannelFrequencies, outChannelIndex, freqTemplate);
+				makeImagingTableEntry(_inputChannelFrequencies, outputIntervalIndex, outChannelIndex, freqTemplate);
 				
 				addPolarizationsToImagingTable(joinedGroupIndex, squaredGroupIndex, outChannelIndex, freqTemplate);
 			}
@@ -1319,7 +1356,7 @@ void WSClean::makeImagingTable(size_t outputIntervalIndex)
 	_imagingTable.Print();
 }
 
-void WSClean::makeImagingTableEntry(const std::vector<OrderedChannel>& channels, size_t outChannelIndex, ImagingTableEntry& entry)
+void WSClean::makeImagingTableEntry(const std::vector<OrderedChannel>& channels, size_t outIntervalIndex, size_t outChannelIndex, ImagingTableEntry& entry)
 {
 	size_t startCh, width;
 	if(_settings.endChannel != 0)
@@ -1343,11 +1380,12 @@ void WSClean::makeImagingTableEntry(const std::vector<OrderedChannel>& channels,
 	entry.highestFrequency = channels[chHighIndex].Frequency();
 	entry.bandStartFrequency = entry.lowestFrequency - channels[chLowIndex].Width()*0.5;
 	entry.bandEndFrequency = entry.highestFrequency + channels[chHighIndex].Width()*0.5;
+	entry.outputIntervalIndex = outIntervalIndex;
 	
 	entry.msData.resize(_settings.filenames.size());
 	for(size_t msIndex=0; msIndex!=_settings.filenames.size(); ++msIndex)
 	{
-		entry.msData[msIndex].bands.resize(_msBands[msIndex].BandCount());
+		entry.msData[msIndex].bands.resize(_msBands[msIndex].DataDescCount());
 	}
 }
 
@@ -1389,11 +1427,23 @@ void WSClean::fitBeamSize(double& bMaj, double& bMin, double& bPA, const double*
 	GaussianFitter beamFitter;
 	Logger::Info << "Fitting beam... ";
 	Logger::Info.Flush();
-	beamFitter.Fit2DGaussianCentred(
-		image,
-		_settings.trimmedImageWidth, _settings.trimmedImageHeight,
-		beamEstimate,
-		bMaj, bMin, bPA);
+	if(_settings.circularBeam)
+	{
+		bMaj = beamEstimate;
+		beamFitter.Fit2DCircularGaussianCentred(
+			image,
+			_settings.trimmedImageWidth, _settings.trimmedImageHeight,
+			bMaj);
+		bMin = bMaj;
+		bPA = 0.0;
+	}
+	else {
+		beamFitter.Fit2DGaussianCentred(
+			image,
+			_settings.trimmedImageWidth, _settings.trimmedImageHeight,
+			beamEstimate,
+			bMaj, bMin, bPA);
+	}
 	if(bMaj < 1.0) bMaj = 1.0;
 	if(bMin < 1.0) bMin = 1.0;
 	bMaj = bMaj*0.5*(_settings.pixelScaleX+_settings.pixelScaleY);
@@ -1413,12 +1463,6 @@ void WSClean::determineBeamSize(double& bMaj, double& bMin, double& bPA, const d
 		Logger::Info << "major=" << Angle::ToNiceString(bMaj) << ", minor=" <<
 		Angle::ToNiceString(bMin) << ", PA=" << Angle::ToNiceString(bPA) << ", theoretical=" <<
 		Angle::ToNiceString(theoreticBeam)<< ".\n";
-		
-		if(_settings.circularBeam)
-		{
-			bMin = bMaj;
-			bPA = 0.0;
-		}
 	}
 	else if(_settings.theoreticBeam) {
 		bMaj = theoreticBeam;
@@ -1439,7 +1483,5 @@ WSCFitsWriter WSClean::createWSCFitsWriter(const ImagingTableEntry& entry, bool
 
 WSCFitsWriter WSClean::createWSCFitsWriter(const ImagingTableEntry& entry, PolarizationEnum polarization, bool isImaginary) const
 {
-	WSCFitsWriter writer(createWSCFitsWriter(entry, isImaginary));
-	writer.SetPolarization(polarization);
-	return writer;
+	return WSCFitsWriter(entry, polarization, isImaginary, _settings, _deconvolution, _majorIterationNr, *_gridder, _commandLine, _infoPerChannel[entry.outputChannelIndex]);
 }
