@@ -35,11 +35,11 @@ public:
 	
 	void RunPredict();
 private:
-	void runIndependentGroup(const ImagingTable& groupTable);
+	void runIndependentGroup(ImagingTable& groupTable);
 	void saveRestoredImagesForGroup(const ImagingTableEntry& tableEntry) const;
 	void predictGroup(const ImagingTable& imagingGroup);
 	
-	void runFirstInversion(const ImagingTableEntry& entry);
+	void runFirstInversion(ImagingTableEntry& entry);
 	void prepareInversionAlgorithm(PolarizationEnum polarization);
 	
 	void performReordering(bool isPredictMode);
@@ -56,12 +56,12 @@ private:
 	void readEarlierModelImages(const ImagingTableEntry& entry);
 	
 	void makeImagingTable(size_t outputIntervalIndex);
-	void makeImagingTableEntry(const std::vector<OrderedChannel>& channels, size_t outChannelIndex, ImagingTableEntry& entry);
+	void makeImagingTableEntry(const std::vector<OrderedChannel>& channels, size_t outIntervalIndex, size_t outChannelIndex, ImagingTableEntry& entry);
 	void addPolarizationsToImagingTable(size_t& joinedGroupIndex, size_t& squaredGroupIndex, size_t outChannelIndex, const ImagingTableEntry& templateEntry);
 	class ImageWeightCache* createWeightCache();
 	
 	void multiplyImage(double factor, double* image) const;
-	void imagePSF(const ImagingTableEntry& entry);
+	void imagePSF(ImagingTableEntry& entry);
 	void imageGridding();
 	void imageMainFirst(PolarizationEnum polarization, size_t channelIndex);
 	void imageMainNonFirst(PolarizationEnum polarization, size_t channelIndex);
@@ -76,6 +76,7 @@ private:
 	
 	void fitBeamSize(double& bMaj, double& bMin, double& bPA, const double* image, double beamEstimate) const;
 	void determineBeamSize(double& bMaj, double& bMin, double& bPA, const double* image, double theoreticBeam) const;
+	double minTheoreticalBeamSize(const ImagingTable& table) const;
 	
 	void makeBeam();
 	
